@@ -44,7 +44,7 @@ class AmbariPluginProvider(p.ProvisioningPluginBase):
                  "using Apache Ambari")
 
     def get_versions(self):
-        return ["2.3", "2.4", "2.5"]
+        return ["2.3", "2.4", "2.5", "2.6"]
 
     def get_node_processes(self, hadoop_version):
         return {
@@ -276,11 +276,18 @@ class AmbariPluginProvider(p.ProvisioningPluginBase):
 
     def pack_image(self, hadoop_version, remote,
                    test_only=False, image_arguments=None):
+        if hadoop_version == '2.3':
+            image_arguments['ambari_version'] = '2.4.3.0'
+
         self.validator.validate(remote, test_only=test_only,
                                 image_arguments=image_arguments)
 
     def validate_images(self, cluster, test_only=False, image_arguments=None):
         image_arguments = self.get_image_arguments(cluster['hadoop_version'])
+        if cluster['hadoop_version'] == '2.3':
+            for arguments in image_arguments:
+                if arguments.name == 'ambari_version':
+                    arguments.default = '2.4.3.0'
         if not test_only:
             instances = plugin_utils.get_instances(cluster)
         else:
