@@ -58,7 +58,7 @@ class ConfigHelper(object):
                      " will be downloaded by VMs."))
 
     DEFAULT_EXTJS_LIB_URL = (
-        'http://tarballs.openstack.org/sahara/dist/common-artifacts/'
+        'https://tarballs.openstack.org/sahara-extra/dist/common-artifacts/'
         'ext-2.2.zip')
 
     EXTJS_LIB_URL = p.Config(
@@ -291,6 +291,9 @@ class ConfigHelper(object):
         return self._get_config_value(cluster,
                                       self.ENABLE_HBASE_COMMON_LIB)
 
+    def is_keytrustee_available(self):
+        return True
+
     def get_swift_lib_url(self, cluster):
         return self._get_config_value(cluster, self.SWIFT_LIB_URL)
 
