@@ -22,6 +22,7 @@ from sahara import conductor as cond
 from sahara import context
 from sahara import exceptions as exc
 from sahara.plugins import base as pl_base
+from sahara.plugins import utils as u
 from sahara.service import api as service_api
 from sahara.service.api.v2 import clusters as api
 from sahara.tests.unit import base
@@ -74,6 +75,21 @@ class FakeOps(object):
             conductor.node_group_update(context.ctx(), ng, {'count': count})
         conductor.cluster_update(context.ctx(), id, {'status': 'Scaled'})
 
+    def update_keypair(self, id):
+        self.calls_order.append('ops.update_keypair')
+        cluster = conductor.cluster_get(context.ctx(), id)
+        keypair_name = cluster.user_keypair_id
+        nova_p = mock.patch("sahara.utils.openstack.nova.client")
+        nova = nova_p.start()
+        key = nova.get_keypair(keypair_name)
+        nodes = u.get_instances(cluster)
+        for instance in nodes:
+            remote = mock.Mock()
+            remote.execute_command(
+                "echo {keypair} >> ~/.ssh/authorized_keys".format(
+                    keypair=key.public_key))
+            remote.reset_mock()
+
     def terminate_cluster(self, id, force):
         self.calls_order.append('ops.terminate_cluster')
 
@@ -123,8 +139,10 @@ class TestClusterApi(base.SaharaWithDbTestCase):
         MULTIPLE_CLUSTERS['count'] = 2
         clusters = api.create_multiple_clusters(MULTIPLE_CLUSTERS)
         self.assertEqual(2, check_cluster.call_count)
-        result_cluster1 = api.get_cluster(clusters['clusters'][0])
-        result_cluster2 = api.get_cluster(clusters['clusters'][1])
+        result_cluster1 = api.get_cluster(
+            clusters['clusters'][0]['cluster']['id'])
+        result_cluster2 = api.get_cluster(
+            clusters['clusters'][1]['cluster']['id'])
         self.assertEqual(c_u.CLUSTER_STATUS_ACTIVE, result_cluster1.status)
         self.assertEqual(c_u.CLUSTER_STATUS_ACTIVE, result_cluster2.status)
         expected_count = {
@@ -226,6 +244,37 @@ class TestClusterApi(base.SaharaWithDbTestCase):
              'ops.provision_scaled_cluster',
              'ops.terminate_cluster'], self.calls_order)
 
+    @mock.patch('sahara.service.quotas.check_cluster', return_value=None)
+    @mock.patch('sahara.service.quotas.check_scaling', return_value=None)
+    def test_scale_cluster_specific_and_non_specific(self, check_scaling,
+                                                     check_cluster):
+        cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
+        cluster = api.get_cluster(cluster.id)
+        api.scale_cluster(cluster.id, api_base.SCALE_DATA_SPECIFIC_INSTANCE)
+        result_cluster = api.get_cluster(cluster.id)
+        self.assertEqual('Scaled', result_cluster.status)
+        expected_count = {
+            'ng_1': 3,
+            'ng_2': 1,
+            'ng_3': 1,
+        }
+        ng_count = 0
+        for ng in result_cluster.node_groups:
+            self.assertEqual(expected_count[ng.name], ng.count)
+            ng_count += 1
+        self.assertEqual(1, result_cluster.node_groups[1].count)
+        self.assertNotIn('ng_2_0',
+                         self._get_instances_ids(
+                             result_cluster.node_groups[1]))
+        self.assertEqual(3, ng_count)
+        api.terminate_cluster(result_cluster.id)
+        self.assertEqual(
+            ['get_open_ports', 'recommend_configs', 'validate',
+             'ops.provision_cluster', 'get_open_ports',
+             'recommend_configs', 'validate_scaling',
+             'ops.provision_scaled_cluster',
+             'ops.terminate_cluster'], self.calls_order)
+
     def _get_instances_ids(self, node_group):
         instance_ids = []
         for instance in node_group.instances:
@@ -247,3 +296,8 @@ class TestClusterApi(base.SaharaWithDbTestCase):
             updated_cluster = api.update_cluster(
                 cluster.id, {'description': 'Cluster'})
             self.assertEqual('Cluster', updated_cluster.description)
+
+    def test_cluster_keypair_update(self):
+        with mock.patch('sahara.service.quotas.check_cluster'):
+            cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
+            api.update_cluster(cluster.id, {'update_keypair': True})
