@@ -73,7 +73,7 @@ ssh_config_options = [
         'ssh_timeout_interactive', default=1800, min=1,
         help="Overrides timeout for interactive ssh operations, in seconds"),
     cfg.IntOpt(
-        'ssh_timeout_files', default=120, min=1,
+        'ssh_timeout_files', default=600, min=1,
         help="Overrides timeout for ssh operations with files, in seconds"),
 ]
 
@@ -270,9 +270,8 @@ def _get_http_client(host, port, proxy_command=None, gateway_host=None,
 
 
 def _write_fl(sftp, remote_file, data):
-    fl = sftp.file(remote_file, 'w')
-    fl.write(data)
-    fl.close()
+    write_data = paramiko.py3compat.StringIO(data)
+    sftp.putfo(write_data, remote_file)
 
 
 def _append_fl(sftp, remote_file, data):
@@ -302,19 +301,6 @@ def _append_file(sftp, remote_file, data, run_as_root):
         _append_fl(sftp, remote_file, data)
 
 
-def _prepend_file(sftp, remote_file, data, run_as_root):
-    if run_as_root:
-        temp_file = 'temp-file-%s' % uuidutils.generate_uuid()
-        temp_remote_file = 'temp-remote-file-%s' % uuidutils.generate_uuid()
-        _write_fl(sftp, temp_file, data)
-        _execute_command(
-            'cat %s > %s' % (remote_file, temp_remote_file))
-        _execute_command(
-            'cat %s %s > %s' % (
-                temp_file, temp_remote_file, remote_file), run_as_root=True)
-        _execute_command('rm -f %s %s' % (temp_file, temp_remote_file))
-
-
 def _write_file_to(remote_file, data, run_as_root=False):
     global _ssh
 
@@ -345,21 +331,6 @@ def _append_to_files(files, run_as_root=False):
         _append_file(sftp, fl, data, run_as_root)
 
 
-def _prepend_to_file(remote_file, data, run_as_root=False):
-    global _ssh
-
-    _prepend_file(_ssh.open_sftp(), remote_file, data, run_as_root)
-
-
-def _prepend_to_files(files, run_as_root=False):
-    global _ssh
-
-    sftp = _ssh.open_sftp()
-
-    for fl, data in six.iteritems(files):
-        _prepend_file(sftp, fl, data, run_as_root)
-
-
 def _read_file(sftp, remote_file):
     fl = sftp.file(remote_file, 'r')
     data = fl.read()
@@ -458,6 +429,13 @@ def _replace_remote_string(remote_file, old_str, new_str):
     _execute_command(cmd)
 
 
+def _replace_remote_line(remote_file, old_line_with_start_string, new_line):
+    search_string = old_line_with_start_string.replace("\'", "\''")
+    cmd = ("sudo sed -i  's/^%s.*/%s/' %s" % (search_string,
+                                              new_line, remote_file))
+    _execute_command(cmd)
+
+
 def _execute_on_vm_interactive(cmd, matcher):
     global _ssh
 
@@ -900,12 +878,6 @@ class InstanceInteropHelper(remote.Remote):
         self._log_command(description)
         self._run_s(_append_to_files, timeout, description, files, run_as_root)
 
-    def prepend_to_file(self, r_file, data, run_as_root=False, timeout=None):
-        description = _('Prepending to file "%s"') % r_file
-        self._log_command(description)
-        self._run_s(_prepend_to_file, timeout, description,
-                    r_file, data, run_as_root)
-
     def read_file_from(self, remote_file, run_as_root=False, timeout=None):
         description = _('Reading file "%s"') % remote_file
         self._log_command(description)
@@ -936,12 +908,28 @@ class InstanceInteropHelper(remote.Remote):
                               timeout=None):
         description = _('In file "%(file)s" replacing string '
                         '"%(old_string)s" with "%(new_string)s"') % {
-            "file": remote_file, "old_string": old_str, "new_string": new_str}
+            "file": remote_file,
+            "old_string": old_str, "new_string": new_str}
 
         self._log_command(description)
         self._run_s(_replace_remote_string, timeout, description,
                     remote_file, old_str, new_str)
 
+    def replace_remote_line(self, remote_file,
+                            old_line_with_start_string,
+                            new_line, timeout=None):
+        description = _('In file "%(file)s" replacing line'
+                        ' begining with string'
+                        '"%(old_line_with_start_string)s"'
+                        ' with "%(new_line)s"') % {
+            "file": remote_file,
+            "old_line_with_start_string": old_line_with_start_string,
+            "new_line": new_line}
+
+        self._log_command(description)
+        self._run_s(_replace_remote_line, timeout, description,
+                    remote_file, old_line_with_start_string, new_line)
+
     def execute_on_vm_interactive(self, cmd, matcher, timeout=None):
         """Runs given command and responds to prompts.
 
